@@ -1,4 +1,4 @@
-use strict;
+use 5.014;
 use warnings;
 use DBD::SQLite 1.31;
 
@@ -9,17 +9,14 @@ package # hide from PAUSE
 use Carp 'croak';
 
 sub _get_table_schema {
-    my($class, $me, $schema, $table) = @_;
+    my($class, $me, $table) = @_;
 
-    my $q_schema = $schema;
-    my $q_table = $table;
-    $q_schema =~ s/([\\_%])/\\$1/g if defined $q_schema;
-    $q_table =~ s/([\\_%])/\\$1/g;
+    my $q_table = $table =~ s/([\\_%])/\\$1/gr;
 
     # Try just these types
-    my $info = $me->rdbh->table_info(undef, $q_schema, $q_table,
+    my $info = $me->rdbh->table_info(undef, undef, $q_table,
         'TABLE,VIEW,GLOBAL TEMPORARY,LOCAL TEMPORARY,SYSTEM TABLE', {Escape => '\\'})->fetchall_arrayref;
-    croak 'Invalid table: '.$class->_qi($me, $schema, $table) unless $info and @$info == 1 and $info->[0][2] eq $table;
+    croak 'Invalid table: '.$class->_qi($me, $table) unless $info and @$info == 1 and $info->[0][2] eq $table;
     return $info->[0][1];
 }
 
@@ -28,4 +25,14 @@ sub _save_last_insert_id {
     $sth->{Database}->last_insert_id(undef, @$me{qw(Schema Name)}, undef);
 }
 
+sub _build_limit {
+    my($class, $me) = @_;
+    my $h = $me->_build_data;
+    return '' unless defined $h->{limit};
+    my $sql = 'LIMIT ';
+    $sql .= $h->{limit}[0] >= 0 ? $h->{limit}[0] : -1;
+    $sql .= ' OFFSET '.$h->{limit}[1] if $h->{limit}[1];
+    return $sql;
+}
+
 1;
